@@ -1,5 +1,5 @@
 /*
- * $Id: shoutcast.c,v 1.126 2004/01/26 16:42:16 jylefort Exp $
+ * $Id: shoutcast.c,v 1.141 2004/03/28 21:52:14 jylefort Exp $
  *
  * Copyright (c) 2002, 2003, 2004 Jean-Yves Lefort
  * All rights reserved.
@@ -54,8 +54,17 @@ typedef struct
   int		listeners;
   int		max;
   int		bitrate;
-
   char		*url_postfix;
+  char		*homepage;
+
+  GSList	*url_list;       /* will be read and written in threads... */
+  GMutex	*url_list_mutex; /* ...so we protect it                    */
+  /*
+   * We don't protect the other stream members because they are
+   * guaranteed to never be accessed concurrently
+   * (ST_HANDLER_EVENT_STREAM_FIELD_SET isn't a problem because it is
+   * only called once, at streamtuner initialization time).
+   */
 } SHOUTcastStream;
 
 enum {
@@ -65,7 +74,9 @@ enum {
   FIELD_LISTENERS,
   FIELD_MAX,
   FIELD_BITRATE,
-  FIELD_URL_POSTFIX
+  FIELD_URL_POSTFIX,
+  FIELD_HOMEPAGE,
+  FIELD_URL_LIST
 };
 
 typedef struct
@@ -121,14 +132,19 @@ static void stream_field_set_cb (SHOUTcastStream	*stream,
 				 STHandlerField		*field,
 				 const GValue		*value,
 				 gpointer		data);
+static void stream_stock_field_get_cb (SHOUTcastStream		*stream,
+				       STHandlerStockField	stock_field,
+				       GValue			*value,
+				       gpointer			data);
 static void stream_free_cb	(SHOUTcastStream	*stream,
 				 gpointer		data);
 
-static gboolean url_list_get		(SHOUTcastStream	*stream,
-					 GSList			**url_list,
+static gboolean stream_resolve		(SHOUTcastStream	*stream,
 					 GError			**err);
-static void     url_list_free		(GSList			*url_list);
 
+static gboolean stream_resolve_cb	(SHOUTcastStream	*stream,
+					 gpointer		data,
+					 GError			**err);
 static gboolean	stream_tune_in_cb	(SHOUTcastStream	*stream,
 					 gpointer		data,
 					 GError			**err);
@@ -149,7 +165,12 @@ static void	init_handler	(void);
 static SHOUTcastStream *
 stream_new_cb (gpointer data)
 {
-  return g_new0(SHOUTcastStream, 1);
+  SHOUTcastStream *stream;
+
+  stream = g_new0(SHOUTcastStream, 1);
+  stream->url_list_mutex = g_mutex_new();
+
+  return stream;
 }
 
 static void
@@ -188,8 +209,33 @@ stream_field_get_cb (SHOUTcastStream *stream,
       g_value_set_string(value, stream->url_postfix);
       break;
 
+    case FIELD_HOMEPAGE:
+      g_value_set_string(value, stream->homepage);
+      break;
+
+    case FIELD_URL_LIST:
+      {
+	GValueArray *value_array = g_value_array_new(0);
+	GSList *l;
+
+	g_mutex_lock(stream->url_list_mutex);
+	for (l = stream->url_list; l; l = l->next)
+	  {
+	    GValue url_value = { 0, };
+
+	    g_value_init(&url_value, G_TYPE_STRING);
+	    g_value_set_string(&url_value, l->data);
+	    g_value_array_append(value_array, &url_value);
+	    g_value_unset(&url_value);
+	  }
+	g_mutex_unlock(stream->url_list_mutex);
+
+	g_value_set_boxed_take_ownership(value, value_array);
+	break;
+      }
+      
     default:
-      g_assert_not_reached();
+      g_return_if_reached();
     }
 }
 
@@ -229,32 +275,98 @@ stream_field_set_cb (SHOUTcastStream *stream,
       stream->url_postfix = g_value_dup_string(value);
       break;
 
+    case FIELD_HOMEPAGE:
+      stream->homepage = g_value_dup_string(value);
+      break;
+
+    case FIELD_URL_LIST:
+      {
+	GValueArray *value_array;
+	int i;
+
+	value_array = g_value_get_boxed(value);
+
+	g_mutex_lock(stream->url_list_mutex);
+	for (i = 0; i < value_array->n_values; i++)
+	  {
+	    GValue *url_value = g_value_array_get_nth(value_array, i);
+	    stream->url_list = g_slist_append(stream->url_list, g_value_dup_string(url_value));
+	  }
+	g_mutex_unlock(stream->url_list_mutex);
+
+	break;
+      }
+      
     default:
-      g_assert_not_reached();
+      g_return_if_reached();
+    }
+}
+
+static void
+stream_stock_field_get_cb (SHOUTcastStream *stream,
+			   STHandlerStockField stock_field,
+			   GValue *value,
+			   gpointer data)
+{
+  switch (stock_field)
+    {
+    case ST_HANDLER_STOCK_FIELD_NAME:
+      g_value_set_string(value, stream->description);
+      break;
+
+    case ST_HANDLER_STOCK_FIELD_GENRE:
+      g_value_set_string(value, stream->genre);
+      break;
+
+    default:
+      /*
+       * This can not happen.
+       *
+       * However, handlers not shipped with streamtuner MUST ignore
+       * unknown stock fields, to stay compatible with future versions
+       * of streamtuner.
+       */
+      g_return_if_reached();
     }
 }
 
 static void
 stream_free_cb (SHOUTcastStream *stream, gpointer data)
 {
+  GSList *l;
+
   g_free(stream->genre);
   g_free(stream->description);
   g_free(stream->now_playing);
   g_free(stream->url_postfix);
+  g_free(stream->homepage);
+
+  for (l = stream->url_list; l; l = l->next)
+    g_free(l->data);
+
+  g_slist_free(stream->url_list);
+  g_mutex_free(stream->url_list_mutex);
 
   st_stream_free((STStream *) stream);
 }
 
 static gboolean
-url_list_get (SHOUTcastStream *stream, GSList **url_list, GError **err)
+stream_resolve (SHOUTcastStream *stream, GError **err)
 {
+  gboolean already_resolved;
   STTransferSession *session;
   char *url;
   char *playlist;
   gboolean status;
 
   g_return_val_if_fail(stream != NULL, FALSE);
-  g_return_val_if_fail(url_list != NULL, FALSE);
+
+  g_mutex_lock(stream->url_list_mutex);
+  already_resolved = stream->url_list != NULL;
+  g_mutex_unlock(stream->url_list_mutex);
+
+  if (already_resolved)
+    return TRUE;		/* already resolved */
 
   url = g_strconcat(SHOUTCAST_ROOT, stream->url_postfix, NULL);
   session = st_transfer_session_new();
@@ -264,10 +376,16 @@ url_list_get (SHOUTcastStream *stream, GSList **url_list, GError **err)
 
   if (status)
     {
-      *url_list = st_pls_parse(playlist);
+      gboolean empty;
+
+      g_mutex_lock(stream->url_list_mutex);
+      stream->url_list = st_pls_parse(playlist);
+      empty = stream->url_list == NULL;
+      g_mutex_unlock(stream->url_list_mutex);
+
       g_free(playlist);
 
-      if (! *url_list)
+      if (empty)
 	{
 	  g_set_error(err, 0, 0, _("stream is empty"));
 	  return FALSE;
@@ -277,47 +395,28 @@ url_list_get (SHOUTcastStream *stream, GSList **url_list, GError **err)
   return status;
 }
 
-static void
-url_list_free (GSList *url_list)
+static gboolean
+stream_resolve_cb (SHOUTcastStream *stream, gpointer data, GError **err)
 {
-  GSList *l;
-
-  for (l = url_list; l; l = l->next)
-    g_free(l->data);
-
-  g_slist_free(url_list);
+  return stream_resolve(stream, err);
 }
 
 static gboolean
-stream_tune_in_cb (SHOUTcastStream *stream,
-		   gpointer data,
-		   GError **err)
+stream_tune_in_cb (SHOUTcastStream *stream, gpointer data, GError **err)
 {
-  GSList *url_list;
   char *m3uname;
   gboolean status;
   
-  /* fetch the URL list */
-
-  if (! url_list_get(stream, &url_list, err))
+  if (! stream_resolve(stream, err))
     return FALSE;
 
-  if (st_is_aborted())
-    {
-      url_list_free(url_list);
-      return FALSE;
-    }
-
-  /* write the .m3u file */
-
-  m3uname = st_m3u_mktemp("streamtuner.shoutcast.XXXXXX", url_list, err);
-  url_list_free(url_list);
+  g_mutex_lock(stream->url_list_mutex);
+  m3uname = st_m3u_mktemp("streamtuner.shoutcast.XXXXXX", stream->url_list, err);
+  g_mutex_unlock(stream->url_list_mutex);
 
   if (! m3uname)
     return FALSE;
-
-  /* pass the .m3u file to the player */
-
+  
   status = st_action_run("play-m3u", m3uname, err);
   g_free(m3uname);
 
@@ -325,57 +424,30 @@ stream_tune_in_cb (SHOUTcastStream *stream,
 }
 
 static gboolean
-stream_record_cb (SHOUTcastStream *stream,
-		  gpointer data,
-		  GError **err)
+stream_record_cb (SHOUTcastStream *stream, gpointer data, GError **err)
 {
-  GSList *url_list;
   gboolean status;
 
-  /* fetch the URL list */
-
-  if (! url_list_get(stream, &url_list, err))
+  if (! stream_resolve(stream, err))
     return FALSE;
 
-  if (st_is_aborted())
-    {
-      url_list_free(url_list);
-      return FALSE;
-    }
-  
-  /* pass the first URL to the recorder */
-
-  status = st_action_run("record-stream", url_list->data, err);
-  url_list_free(url_list);
+  g_mutex_lock(stream->url_list_mutex);
+  status = st_action_run("record-stream", stream->url_list->data, err);
+  g_mutex_unlock(stream->url_list_mutex);
 
   return status;
 }
 
 static gboolean
-stream_browse_cb (SHOUTcastStream *stream,
-		  gpointer data,
-		  GError **err)
+stream_browse_cb (SHOUTcastStream *stream, gpointer data, GError **err)
 {
-  GSList *url_list;
-  gboolean status;
-
-  /* fetch the URL list */
-
-  if (! url_list_get(stream, &url_list, err))
-    return FALSE;
-
-  if (st_is_aborted())
+  if (! stream->homepage) /* older versions of the plugin didn't have this field */
     {
-      url_list_free(url_list);
+      g_set_error(err, 0, 0, _("the stream is too old, please refresh"));
       return FALSE;
     }
 
-  /* pass the first URL to the browser */
-
-  status = st_action_run("view-web", url_list->data, err);
-  url_list_free(url_list);
-
-  return status;
+  return st_action_run("view-web", stream->homepage, err);
 }
 
 static gboolean
@@ -486,17 +558,22 @@ static void
 refresh_header_cb (const char *line, gpointer data)
 {
   RefreshInfo *info = data;
+  char *charset;
 
-  if (! info->charset)
-    st_re_parse(&re_header_charset, line, &info->charset);
+  if (st_re_parse(&re_header_charset, line, &charset))
+    {
+      g_free(info->charset);
+      info->charset = charset;
+    }
 }
 
 static void
 refresh_body_cb (const char *line, gpointer data)
 {
   RefreshInfo *info = data;
-  char *sub1, *sub2;
+  char *sub1, *sub2, *sub3;
   char *converted = NULL;
+  char *expanded;
 
   if (! info->charset && ! strncasecmp(line, "</head>", 6))
     info->charset = g_strdup("ISO8859-1"); /* we got no charset info, use a fallback */
@@ -533,6 +610,9 @@ refresh_body_cb (const char *line, gpointer data)
 	}
     }
   
+  expanded = st_sgml_ref_expand(line);
+  line = expanded;
+
   if (st_re_parse(&re_stream, line, &sub1))
     {
       if (info->stream)	/* a malformed stream remains, free it */
@@ -550,7 +630,7 @@ refresh_body_cb (const char *line, gpointer data)
 	{
 	  info->stream->bitrate = atoi(sub1);
 	  
-	  if (info->stream->genre && info->stream->description)
+	  if (info->stream->genre && info->stream->description && info->stream->homepage)
 	    {
 	      ((STStream *) info->stream)->name =
 		g_strdup_printf("%s%s%i",
@@ -615,18 +695,20 @@ refresh_body_cb (const char *line, gpointer data)
 	  info->parent_node = node;
 	}
     }
-  else if (st_re_parse(&re_genre, line, &sub1, &sub2))
+  else if (st_re_parse(&re_genre, line, &sub1, &sub2, &sub3))
     {
       if (info->stream)
 	{
 	  info->stream->genre = sub1;
-	  info->stream->description = sub2;
+	  info->stream->homepage = sub2;
+	  info->stream->description = sub3;
 	}
       else
 	{
-	  st_notice(_("SHOUTcast: found misplaced genre and description"));
+	  st_notice(_("SHOUTcast: found misplaced genre, homepage and description"));
 	  g_free(sub1);
 	  g_free(sub2);
+	  g_free(sub3);
 	}
     }
   else if (st_re_parse(&re_playing, line, &sub1))
@@ -662,8 +744,8 @@ refresh_body_cb (const char *line, gpointer data)
       g_free(sub2);
     }
   
-  if (converted)
-    g_free(converted);
+  g_free(converted);
+  g_free(expanded);
 }
 
 static gboolean
@@ -701,7 +783,7 @@ init_re (void)
   g_return_val_if_fail(status == 0, FALSE);
   status = regcomp(&re_body_charset, "<meta http-equiv=.* content=.*charset=(.*)\"", REG_EXTENDED | REG_ICASE);
   g_return_val_if_fail(status == 0, FALSE);
-  status = regcomp(&re_genre, "^      <td .*<b>\\[(.*)\\]</font>.*<a .*>(.*)</a>", REG_EXTENDED);
+  status = regcomp(&re_genre, "^      <td .*<b>\\[(.*)\\]</font>.*<a .*href=\"(.*)\">(.*)</a>", REG_EXTENDED);
   g_return_val_if_fail(status == 0, FALSE);
   status = regcomp(&re_playing, "^<font .*>Now Playing:</font>(.*)</font></font></td>", REG_EXTENDED);
   g_return_val_if_fail(status == 0, FALSE);
@@ -713,7 +795,7 @@ init_re (void)
   g_return_val_if_fail(status == 0, FALSE);
   status = regcomp(&re_category, "^\t\t?<OPTION VALUE=\"(.*)\">(.*)$", REG_EXTENDED);
   g_return_val_if_fail(status == 0, FALSE);
-  status = regcomp(&re_page, "^Page ([0-9]+) of ([0-9]+)&nbsp;", REG_EXTENDED);
+  status = regcomp(&re_page, "^Page ([0-9]+) of ([0-9]+)", REG_EXTENDED);
   g_return_val_if_fail(status == 0, FALSE);
 
   return TRUE;
@@ -746,7 +828,7 @@ init_handler (void)
   handler = st_handler_new("shoutcast");
 
   st_handler_set_label(handler, "SHOUTcast");
-  st_handler_set_copyright(handler, COPYRIGHT);
+  st_handler_set_copyright(handler, "Copyright \302\251 2002, 2003, 2004 Jean-Yves Lefort");
   st_handler_set_description(handler, _("SHOUTcast Yellow Pages"));
   st_handler_set_home(handler, "http://www.shoutcast.com/");
   st_handler_set_icon_from_inline(handler, sizeof(art_shoutcast), art_shoutcast);
@@ -757,8 +839,10 @@ init_handler (void)
   st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_NEW, stream_new_cb, NULL);
   st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_FIELD_GET, stream_field_get_cb, NULL);
   st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_FIELD_SET, stream_field_set_cb, NULL);
+  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_STOCK_FIELD_GET, stream_stock_field_get_cb, NULL);
   st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_FREE, stream_free_cb, NULL);
 
+  st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_RESOLVE, stream_resolve_cb, NULL);
   st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_TUNE_IN, stream_tune_in_cb, NULL);
   st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_RECORD, stream_record_cb, NULL);
   st_handler_bind(handler, ST_HANDLER_EVENT_STREAM_BROWSE, stream_browse_cb, NULL);
@@ -766,31 +850,41 @@ init_handler (void)
   st_handler_add_field(handler, st_handler_field_new(FIELD_GENRE,
 						     _("Genre"),
 						     G_TYPE_STRING,
-						     TRUE));
+						     ST_HANDLER_FIELD_VISIBLE));
   st_handler_add_field(handler, st_handler_field_new(FIELD_DESCRIPTION,
 						     _("Description"),
 						     G_TYPE_STRING,
-						     TRUE));
+						     ST_HANDLER_FIELD_VISIBLE));
   st_handler_add_field(handler, st_handler_field_new(FIELD_NOW_PLAYING,
 						     _("Now playing"),
 						     G_TYPE_STRING,
-						     TRUE));
+						     ST_HANDLER_FIELD_VISIBLE));
   st_handler_add_field(handler, st_handler_field_new(FIELD_LISTENERS,
 						     _("Listeners"),
 						     G_TYPE_INT,
-						     TRUE));
+						     ST_HANDLER_FIELD_VISIBLE));
   st_handler_add_field(handler, st_handler_field_new(FIELD_MAX,
 						     _("Max"),
 						     G_TYPE_INT,
-						     TRUE));
+						     ST_HANDLER_FIELD_VISIBLE));
   st_handler_add_field(handler, st_handler_field_new(FIELD_BITRATE,
 						     _("Bitrate"),
 						     G_TYPE_INT,
-						     TRUE));
+						     ST_HANDLER_FIELD_VISIBLE));
   st_handler_add_field(handler, st_handler_field_new(FIELD_URL_POSTFIX,
 						     _("URL postfix"),
 						     G_TYPE_STRING,
-						     FALSE));
+						     0));
+  st_handler_add_field(handler, st_handler_field_new(FIELD_HOMEPAGE,
+						     _("Homepage"),
+						     G_TYPE_STRING,
+						     ST_HANDLER_FIELD_VISIBLE
+						     | ST_HANDLER_FIELD_START_HIDDEN));
+  st_handler_add_field(handler, st_handler_field_new(FIELD_URL_LIST,
+						     _("URL list"),
+						     G_TYPE_VALUE_ARRAY,
+						     ST_HANDLER_FIELD_VISIBLE
+						     | ST_HANDLER_FIELD_START_HIDDEN));
 
   st_handlers_add(handler);
 }
@@ -818,7 +912,7 @@ plugin_init (GError **err)
   
   st_action_register("play-m3u", _("Listen to a .m3u file"), "xmms %q");
   st_action_register("record-stream", _("Record a stream"), "xterm -hold -e streamripper %q");
-  st_action_register("view-web", _("Browse a website"), "epiphany %q");
+  st_action_register("view-web", _("Open a web page"), "epiphany %q");
 
   return TRUE;
 }
