@@ -1,5 +1,5 @@
 /*
- * $Id: sgtk-util.c,v 1.7 2004/01/27 14:51:15 jylefort Exp $
+ * $Id: sgtk-util.c,v 1.14 2004/03/11 17:55:04 jylefort Exp $
  *
  * Copyright (c) 2003, 2004 Jean-Yves Lefort
  * All rights reserved.
@@ -34,18 +34,17 @@
 #include <gtk/gtk.h>
 #include "gettext.h"
 
+/*** function declarations ***************************************************/
+
+static void sgtk_widget_activate_h (GtkWidget *widget, gpointer data);
+
 /*** implementation **********************************************************/
 
-void sgtk_flush (void)
+void
+sgtk_flush (void)
 {
-  if (gtk_main_level() < 1)
-    GDK_THREADS_ENTER();
-
   while (gtk_events_pending())
     gtk_main_iteration();
-
-  if (gtk_main_level() < 1)
-    GDK_THREADS_LEAVE();
 }
 
 char *
@@ -68,3 +67,28 @@ sgtk_pixbuf_scale (GdkPixbuf *pixbuf, GtkIconSize size)
 
   return gdk_pixbuf_scale_simple(pixbuf, width, height, GDK_INTERP_BILINEAR);
 }
+
+void
+sgtk_widget_set_next_widget (GtkWidget *widget, GtkWidget *next)
+{
+  g_return_if_fail(GTK_IS_WIDGET(widget));
+  g_return_if_fail(GTK_IS_WIDGET(next));
+
+  g_object_set_data(G_OBJECT(widget), "focus-next", next);
+  g_signal_connect(G_OBJECT(widget), "activate",
+		   G_CALLBACK(sgtk_widget_activate_h), next);
+}
+
+static void
+sgtk_widget_activate_h (GtkWidget *widget, gpointer data)
+{
+  GtkWidget *next = data;
+
+  while (! GTK_WIDGET_IS_SENSITIVE(next))
+    {
+      next = g_object_get_data(G_OBJECT(next), "focus-next");
+      g_return_if_fail(next != NULL);
+    }
+
+  gtk_widget_grab_focus(next);
+}
