@@ -1,5 +1,5 @@
 /*
- * $Id: sgtk-util.h,v 1.6 2004/01/27 14:51:15 jylefort Exp $
+ * $Id: sgtk-util.h,v 1.13 2004/03/13 18:51:01 jylefort Exp $
  *
  * Copyright (c) 2003, 2004 Jean-Yves Lefort
  * All rights reserved.
@@ -34,10 +34,20 @@
 #ifndef _SGTK_UTIL_H
 #define _SGTK_UTIL_H
 
-void		sgtk_flush		(void);
-char		*sgtk_translate_func	(const char	*path,
-					 gpointer	data);
-GdkPixbuf	*sgtk_pixbuf_scale	(GdkPixbuf	*pixbuf,
-					 GtkIconSize	size);
+#include <gtk/gtk.h>
+
+typedef enum
+{
+  SGTK_RESPONSE_PREVIOUS,
+  SGTK_RESPONSE_NEXT
+} sGtkResponseType;
+
+void		sgtk_flush			(void);
+char		*sgtk_translate_func		(const char	*path,
+						 gpointer	data);
+GdkPixbuf	*sgtk_pixbuf_scale		(GdkPixbuf	*pixbuf,
+						 GtkIconSize	size);
+void		sgtk_widget_set_next_widget	(GtkWidget	*widget,
+						 GtkWidget	*next);
 
 #endif /* _SGTK_UTIL_H */
