@@ -1,5 +1,5 @@
 /*
- * $Id: st-about-dialog.c,v 1.27 2004/01/28 11:33:54 jylefort Exp $
+ * $Id: st-about-dialog.c,v 1.33 2004/03/19 20:20:13 jylefort Exp $
  *
  * Copyright (c) 2003, 2004 Jean-Yves Lefort
  * All rights reserved.
@@ -33,7 +33,6 @@
 
 #include "config.h"
 #include <gtk/gtk.h>
-#include "art/streamtuner_logo.h"
 #include "gettext.h"
 #include "sgtk-dialog.h"
 #include "sgtk-hig.h"
@@ -43,12 +42,13 @@
 #include "st-link.h"
 #include "st-settings.h"
 #include "sg-util.h"
+#include "st-handler.h"
+#include "st-util.h"
 
 /*** function declarations ***************************************************/
 
 static void      st_about_dialog_init		(STAboutDialog *dialog);
 
-static GtkWidget *st_about_dialog_make_logo	(void);
 static char      *st_about_dialog_make_info	(void);
 
 /*** implementation **********************************************************/
@@ -100,11 +100,11 @@ st_about_dialog_init (STAboutDialog *dialog)
 
   /* create the logo */
 
-  logo = st_about_dialog_make_logo();
+  logo = st_logo_new();
 
   /* create the version and copyright */
 
-  tmp = g_strdup_printf(_("\nversion %s\n%s\n"), VERSION, COPYRIGHT);
+  tmp = g_strdup_printf(_("\nversion %s\n%s\n"), VERSION, ST_COPYRIGHT);
   version = gtk_label_new(tmp);
   g_free(tmp);
 
@@ -114,8 +114,8 @@ st_about_dialog_init (STAboutDialog *dialog)
   /* create the homepage link */
 
   link = st_link_new();
-  st_link_set_text(ST_LINK(link), PACKAGE_HOME);
-  st_link_set_uri(ST_LINK(link), PACKAGE_HOME);
+  st_link_set_text(ST_LINK(link), ST_HOME);
+  st_link_set_uri(ST_LINK(link), ST_HOME);
 
   /* create the dynamic info */
 
@@ -140,32 +140,6 @@ st_about_dialog_init (STAboutDialog *dialog)
   gtk_widget_show_all(box);
 }
 
-static GtkWidget *
-st_about_dialog_make_logo (void)
-{
-  GtkWidget *image;
-  GtkWidget *event_box;
-  GdkPixbuf *logo; /* fff000 */
-  GdkColor bg = { 0, 0xFFFF, 0xF000, 0x0000 };
-
-  logo = gdk_pixbuf_new_from_inline(sizeof(art_streamtuner_logo),
-				    art_streamtuner_logo,
-				    FALSE,
-				    NULL);
-  g_return_val_if_fail(logo != NULL, NULL);
-  
-  image = gtk_image_new_from_pixbuf(logo);
-  g_object_unref(logo);
-
-  event_box = gtk_event_box_new();
-  gtk_widget_modify_bg(event_box, GTK_STATE_NORMAL, &bg);
-  gtk_container_add(GTK_CONTAINER(event_box), image);
-
-  gtk_widget_show(image);
-
-  return event_box;
-}
-
 static char *
 st_about_dialog_make_info (void)
 {
@@ -191,21 +165,19 @@ st_about_dialog_make_info (void)
   else
     g_string_append(string, _("\nThere are no loaded plugins."));
   
-  if (st_handlers_list)
-    {
-      g_string_append(string, _("\n\n<span weight=\"bold\">Registered handlers:</span>\n"));
+  g_string_append(string, _("\n\n<span weight=\"bold\">Registered handlers:</span>\n"));
       
-      SG_LIST_FOREACH(l, st_handlers_list)
-	{
-	  STHandler *handler = l->data;
-	  g_string_append_printf(string, "\n%s", handler->label);
+  SG_LIST_FOREACH(l, st_handlers_list)
+    {
+      STHandler *handler = l->data;
+      const char *copyright;
+
+      g_string_append_printf(string, "\n%s", st_handler_get_label(handler));
 	  
-	  if (handler->copyright)
-	    g_string_append_printf(string, " - %s", handler->copyright);
-	}
+      copyright = st_handler_get_copyright(handler);
+      if (copyright)
+	g_string_append_printf(string, " - %s", copyright);
     }
-  else
-    g_string_append(string, _("\n\nThere are no registered handlers."));
       
   return g_string_free(string, FALSE);
 }
