@@ -1,5 +1,5 @@
 /*
- * $Id: st-bookmarks.c,v 1.29 2004/03/30 00:45:56 jylefort Exp $
+ * $Id: st-bookmarks.c,v 1.30 2004/03/30 17:14:08 jylefort Exp $
  *
  * Copyright (c) 2004 Jean-Yves Lefort
  * All rights reserved.
@@ -311,11 +311,15 @@ st_bookmarks_value_array_new (STStreamBag *stream_bag)
       SG_LIST_FOREACH(l, st_handler_get_fields(stream_bag->handler))
         {
 	  STHandlerField *field = l->data;
-	  GValue value = { 0, };
 
-	  st_stream_bag_get_field(stream_bag, field, &value);
-	  g_value_array_append(value_array, &value);
-	  g_value_unset(&value);
+	  if (! ST_HANDLER_FIELD_IS_VOLATILE(field))
+	    {
+	      GValue value = { 0, };
+
+	      st_stream_bag_get_field(stream_bag, field, &value);
+	      g_value_array_append(value_array, &value);
+	      g_value_unset(&value);
+	    }
 	}
     }
 
@@ -345,14 +349,19 @@ st_bookmarks_stream_new (STHandler *handler, GValueArray *value_array)
   while (l && i < value_array->n_values)
     {
       STHandlerField *field = l->data;
-      GValue *value;
 
-      value = g_value_array_get_nth(value_array, i);
-      if (G_VALUE_TYPE(value) == st_handler_field_get_type(field))
-	st_stream_bag_set_field(stream, field, value);
+      if (! ST_HANDLER_FIELD_IS_VOLATILE(field))
+	{
+	  GValue *value;
+
+	  value = g_value_array_get_nth(value_array, i);
+	  if (G_VALUE_TYPE(value) == st_handler_field_get_type(field))
+	    st_stream_bag_set_field(stream, field, value);
+
+	  i += 1;
+	}
 
       l = l->next;
-      i += 1;
     }
 
   return stream;
