@@ -1,5 +1,5 @@
 /*
- * $Id: st-cache-load.c,v 1.59 2004/03/26 19:02:43 jylefort Exp $
+ * $Id: st-cache-load.c,v 1.60 2004/03/30 16:55:27 jylefort Exp $
  *
  * Copyright (c) 2002, 2003, 2004 Jean-Yves Lefort
  * All rights reserved.
@@ -62,7 +62,8 @@ enum {
   STATEMENT_HANDLER_CATEGORY_STREAMS,
   STATEMENT_HANDLER_CATEGORY_STREAMS_STREAM,
   STATEMENT_HANDLER_CATEGORY_STREAMS_STREAM_FIELDS,
-  STATEMENT_HANDLER_CATEGORY_STREAMS_STREAM_FIELDS_ARRAY
+  STATEMENT_HANDLER_CATEGORY_STREAMS_STREAM_FIELDS_ARRAY,
+  STATEMENT_HANDLER_CATEGORY_STREAMS_STREAM_FIELDS_ARRAY_ARRAY
 };
 
 typedef struct
@@ -88,9 +89,10 @@ typedef struct
   GSList		*stream_field_iter;
   STHandlerField	*stream_field;
 
-  /* currently parsed value array */
+  /* currently parsed value arrays */
 
   GValueArray		*value_array;
+  GValueArray		*child_value_array;
 } STCacheLoadInfo;
   
 /*** constant definitions ****************************************************/
@@ -156,6 +158,11 @@ static SGParserDefinition cache_definitions[] = {
     STATEMENT_HANDLER_CATEGORY_STREAMS_STREAM_FIELDS_ARRAY,
     "array",		TRUE,		G_TYPE_NONE
   },
+  {
+    STATEMENT_HANDLER_CATEGORY_STREAMS_STREAM_FIELDS_ARRAY,
+    STATEMENT_HANDLER_CATEGORY_STREAMS_STREAM_FIELDS_ARRAY_ARRAY,
+    "array",		TRUE,		G_TYPE_NONE
+  },
   { 0, 0, NULL, 0, 0 }
 };
 
@@ -178,6 +185,10 @@ static void st_cache_load_value_array_begin	(STCacheLoadInfo *info);
 static void st_cache_load_value_array_value	(STCacheLoadInfo *info);
 static void st_cache_load_value_array_end	(STCacheLoadInfo *info);
 
+static void st_cache_load_child_value_array_begin (STCacheLoadInfo *info);
+static void st_cache_load_child_value_array_value (STCacheLoadInfo *info);
+static void st_cache_load_child_value_array_end   (STCacheLoadInfo *info);
+
 /*** implementation **********************************************************/
 
 gboolean
@@ -216,6 +227,10 @@ st_cache_load (const char *filename, GError **err)
 	    case STATEMENT_HANDLER_CATEGORY_STREAMS_STREAM_FIELDS_ARRAY:
 	      st_cache_load_value_array_end(&info);
 	      break;
+
+	    case STATEMENT_HANDLER_CATEGORY_STREAMS_STREAM_FIELDS_ARRAY_ARRAY:
+	      st_cache_load_child_value_array_end(&info);
+	      break;
 	    }
 	}
       
@@ -263,8 +278,11 @@ st_cache_load (const char *filename, GError **err)
 	      break;
 
 	    case STATEMENT_HANDLER_CATEGORY_STREAMS_STREAM_FIELDS_ARRAY:
-	      if (info.handler && info.stream_bag)
-		st_cache_load_value_array_begin(&info);
+	      st_cache_load_value_array_begin(&info);
+	      break;
+
+	    case STATEMENT_HANDLER_CATEGORY_STREAMS_STREAM_FIELDS_ARRAY_ARRAY:
+	      st_cache_load_child_value_array_begin(&info);
 	      break;
 	    }
 	}
@@ -281,6 +299,10 @@ st_cache_load (const char *filename, GError **err)
 	      st_cache_load_value_array_value(&info);
 	      break;
 
+	    case STATEMENT_HANDLER_CATEGORY_STREAMS_STREAM_FIELDS_ARRAY_ARRAY:
+	      st_cache_load_child_value_array_value(&info);
+	      break;
+
 	    default:
 	      sg_parser_warn(info.parser, _("unexpected value"));
 	    }
@@ -522,3 +544,39 @@ st_cache_load_value_array_end (STCacheLoadInfo *info)
 
   info->value_array = NULL;
 }
+
+static void
+st_cache_load_child_value_array_begin (STCacheLoadInfo *info)
+{
+  if (! (info->handler && info->value_array))
+    return;
+
+  info->child_value_array = g_value_array_new(0);
+}
+
+static void
+st_cache_load_child_value_array_value (STCacheLoadInfo *info)
+{
+  if (! (info->handler && info->child_value_array))
+    return;
+
+  g_value_array_append(info->child_value_array, &info->statement->value);
+}
+
+static void
+st_cache_load_child_value_array_end (STCacheLoadInfo *info)
+{
+  GValue value = { 0, };
+  
+  if (! (info->handler && info->child_value_array))
+    return;
+
+  g_value_init(&value, G_TYPE_VALUE_ARRAY);
+  g_value_set_boxed_take_ownership(&value, info->child_value_array);
+
+  st_stream_bag_set_field(info->stream_bag, info->stream_field, &value);
+  g_value_array_append(info->value_array, &value);
+  g_value_unset(&value);
+
+  info->child_value_array = NULL;
+}
