@@ -1,5 +1,5 @@
 /*
- * $Id: st-transfer.c,v 1.46.2.1 2004/02/07 19:22:54 jylefort Exp $
+ * $Id: st-transfer.c,v 1.73 2004/03/26 18:04:57 jylefort Exp $
  *
  * Copyright (c) 2002, 2003, 2004 Jean-Yves Lefort
  * All rights reserved.
@@ -36,13 +36,12 @@
 #include <string.h>
 #include "gettext.h"
 #include "sg-util.h"
-#include "sg-printable.h"
 #include "sgtk-auth-dialog.h"
-#include "st-dialog.h"
+#include "st-dialog-api.h"
 #include "st-settings.h"
 #include "st-transfer.h"
 #include "st-thread.h"
-#include "st-shell.h"
+#include "st-main.h"
 
 #define AGENT_STRING			"streamtuner"
 
@@ -62,15 +61,18 @@
  * libcurl already provides this prototype as curl_write_callback, but
  * we are not gonna rely on it as it's undocumented
  */
-typedef size_t (CurlCallback) (char *buffer,
+typedef size_t (CurlCallback) (const char *buffer,
 			       size_t size,
 			       size_t nitems,
-			       void *data);
+			       gpointer data);
 
 typedef struct
 {
+  STThread			*thread;
+  double			downloaded;
+
   const char			*url;
-  int				flags;
+  unsigned int			flags;
 
   STTransferSession		*session;
   char				error[CURL_ERROR_SIZE];
@@ -81,8 +83,9 @@ typedef struct
   CurlCallback			*body_cb;
   gpointer			body_data;
 
-  char				*proxy_url;
-  char				*proxy_auth_name;
+  curl_proxytype		proxy_type;
+  char				*proxy_server;
+  int				proxy_port;
   char				*proxy_userpwd;
 } STTransfer;
 
@@ -104,39 +107,41 @@ typedef struct
 
 static gboolean st_transfer_perform (STTransfer *transfer, GError **err);
 
-static void st_transfer_set_proxy_settings (STTransfer *transfer);
+static gboolean st_transfer_set_proxy_settings (STTransfer *transfer);
 static void st_transfer_free_proxy_settings (STTransfer *transfer);
 
-static gboolean st_transfer_progress_cb (void *data,
+static char *st_transfer_password_prompt (const char *username);
+
+static gboolean st_transfer_progress_cb (gpointer data,
 					 double dltotal,
 					 double dlnow,
 					 double ultotal,
 					 double ulnow);
-static gboolean st_transfer_password_prompt_cb (void *data,
-						char *prompt,
-						char *buffer,
-						int buflen);
 
 static gboolean st_transfer_session_get_internal (STTransferSession *session,
 						  const char *url,
-						  int flags,
+						  unsigned int flags,
 						  char **headers,
 						  char **body,
 						  GError **err);
 static gboolean st_transfer_session_get_by_line_internal (STTransferSession *session,
 							  const char *url,
-							  int flags,
+							  unsigned int flags,
 							  STTransferLineCallback *header_cb,
 							  gpointer header_data,
 							  STTransferLineCallback *body_cb,
 							  gpointer body_data,
 							  GError **err);
 
-static size_t st_transfer_session_get_cb (char *buffer,
+static size_t st_transfer_session_get_cb (const char *buffer,
 					  size_t size,
 					  size_t nitems,
 					  gpointer data);
-static size_t st_transfer_session_get_by_line_cb (char *buffer,
+static size_t st_transfer_session_get_binary_cb (const char *buffer,
+						 size_t size,
+						 size_t nitems,
+						 gpointer data);
+static size_t st_transfer_session_get_by_line_cb (const char *buffer,
 						  size_t size,
 						  size_t nitems,
 						  gpointer data);
@@ -199,7 +204,7 @@ st_transfer_get_full (const char *url, GError **err)
 gboolean
 st_transfer_session_get (STTransferSession *session,
 			 const char *url,
-			 int flags,
+			 unsigned int flags,
 			 char **headers,
 			 char **body,
 			 GError **err)
@@ -207,13 +212,92 @@ st_transfer_session_get (STTransferSession *session,
   gboolean status;
 
   g_return_val_if_fail(url != NULL, FALSE);
-  g_return_val_if_fail(body != NULL, FALSE);
 
   status = st_transfer_session_get_internal(session, url, flags, headers, body, err);
 
   return status;
 }
 
+/*
+ * Fetches URL as binary data.
+ *
+ * If HEADERS is not null, write a newly-allocated string containing
+ * the HTTP headers at its address.
+ *
+ * A newly-allocated buffer containing the body will be written at the
+ * address pointed to by BODY.
+ *
+ * The length of the body will be written at the address pointed to by
+ * BODY_LEN.
+ *
+ * Returns TRUE in case of success, FALSE in case of abort or error
+ * (if there was an error, ERR will be set).
+ */
+gboolean
+st_transfer_session_get_binary (STTransferSession *session,
+				const char *url,
+				unsigned int flags,
+				char **headers,
+				guint8 **body,
+				unsigned int *body_len,
+				GError **err)
+{
+  STTransfer transfer;
+  GString *header_string;
+  GByteArray *body_array;
+  gboolean status;
+
+  g_return_val_if_fail(session != NULL, FALSE);
+  g_return_val_if_fail(url != NULL, FALSE);
+  
+  transfer.url = url;
+  transfer.flags = flags;
+  transfer.session = session;
+
+  if (headers)
+    {
+      header_string = g_string_new(NULL);
+      transfer.header_cb = st_transfer_session_get_cb;
+      transfer.header_data = header_string;
+    }
+  else
+    {
+      transfer.header_cb = NULL;
+      transfer.header_data = NULL;
+    }
+
+  if (body)
+    {
+      body_array = g_byte_array_new();
+      transfer.body_cb = st_transfer_session_get_binary_cb;
+      transfer.body_data = body_array;
+    }
+  else
+    {
+      transfer.body_cb = NULL;
+      transfer.body_data = NULL;
+    }
+  
+  status = st_transfer_perform(&transfer, err);
+  if (status)
+    {
+      if (headers)
+	*headers = header_string->str;
+      if (body)
+	{
+	  *body = body_array->data;
+	  *body_len = body_array->len;
+	}
+    }
+
+  if (headers)
+    g_string_free(header_string, ! status);
+  if (body)
+    g_byte_array_free(body_array, ! status);
+
+  return status;
+}
+
 /*
  * Deprecated
  */
@@ -236,19 +320,18 @@ st_transfer_get_full_with_session (STTransferSession *session,
 static gboolean
 st_transfer_session_get_internal (STTransferSession *session,
 				  const char *url,
-				  int flags,
+				  unsigned int flags,
 				  char **headers,
 				  char **body,
 				  GError **err)
 {
   STTransfer transfer;
-  GString *header_string = NULL;
+  GString *header_string;
   GString *body_string;
   gboolean status;
 
   g_return_val_if_fail(session != NULL, FALSE);
   g_return_val_if_fail(url != NULL, FALSE);
-  g_return_val_if_fail(body != NULL, FALSE);
   
   transfer.url = url;
   transfer.flags = flags;
@@ -266,21 +349,31 @@ st_transfer_session_get_internal (STTransferSession *session,
       transfer.header_data = NULL;
     }
 
-  body_string = g_string_new(NULL);
-  transfer.body_cb = st_transfer_session_get_cb;
-  transfer.body_data = body_string;
+  if (body)
+    {
+      body_string = g_string_new(NULL);
+      transfer.body_cb = st_transfer_session_get_cb;
+      transfer.body_data = body_string;
+    }
+  else
+    {
+      transfer.body_cb = NULL;
+      transfer.body_data = NULL;
+    }
   
   status = st_transfer_perform(&transfer, err);
   if (status)
     {
       if (headers)
 	*headers = header_string->str;
-      *body = body_string->str;
+      if (body)
+	*body = body_string->str;
     }
 
-  if (header_string)
+  if (headers)
     g_string_free(header_string, ! status);
-  g_string_free(body_string, ! status);
+  if (body)
+    g_string_free(body_string, ! status);
 
   return status;
 }
@@ -310,7 +403,7 @@ st_transfer_get_lines (const char *url,
 gboolean
 st_transfer_session_get_by_line (STTransferSession *session,
 				 const char *url,
-				 int flags,
+				 unsigned int flags,
 				 STTransferLineCallback *header_cb,
 				 gpointer header_data,
 				 STTransferLineCallback *body_cb,
@@ -320,7 +413,6 @@ st_transfer_session_get_by_line (STTransferSession *session,
   gboolean status;
 
   g_return_val_if_fail(url != NULL, FALSE);
-  g_return_val_if_fail(body_cb != NULL, FALSE);
 
   status = st_transfer_session_get_by_line_internal(session, url, flags, header_cb, header_data, body_cb, body_data, err);
 
@@ -347,7 +439,7 @@ st_transfer_get_lines_with_session (STTransferSession *session,
 static gboolean
 st_transfer_session_get_by_line_internal (STTransferSession *session,
 					  const char *url,
-					  int flags,
+					  unsigned int flags,
 					  STTransferLineCallback *header_cb,
 					  gpointer header_data,
 					  STTransferLineCallback *body_cb,
@@ -361,7 +453,6 @@ st_transfer_session_get_by_line_internal (STTransferSession *session,
 
   g_return_val_if_fail(session != NULL, FALSE);
   g_return_val_if_fail(url != NULL, FALSE);
-  g_return_val_if_fail(body_cb != NULL, FALSE);
   
   transfer.url = url;
   transfer.flags = flags;
@@ -383,19 +474,28 @@ st_transfer_session_get_by_line_internal (STTransferSession *session,
       transfer.header_data = NULL;
     }
    
-  body_line_data.transfer = &transfer;
-  body_line_data.line = g_string_new(NULL);
-  body_line_data.line_cb = body_cb;
-  body_line_data.line_cb_data = body_data;
-
-  transfer.body_cb = st_transfer_session_get_by_line_cb;
-  transfer.body_data = &body_line_data;
+  if (body_cb)
+    {
+      body_line_data.transfer = &transfer;
+      body_line_data.line = g_string_new(NULL);
+      body_line_data.line_cb = body_cb;
+      body_line_data.line_cb_data = body_data;
+      
+      transfer.body_cb = st_transfer_session_get_by_line_cb;
+      transfer.body_data = &body_line_data;
+    }
+  else
+    {
+      transfer.body_cb = NULL;
+      transfer.body_data = NULL;
+    }
 
   status = st_transfer_perform(&transfer, err);
   
   if (header_cb)
     g_string_free(header_line_data.line, TRUE);
-  g_string_free(body_line_data.line, TRUE);
+  if (body_cb)
+    g_string_free(body_line_data.line, TRUE);
   
   return status;
 }
@@ -403,34 +503,39 @@ st_transfer_session_get_by_line_internal (STTransferSession *session,
 static gboolean
 st_transfer_perform (STTransfer *transfer, GError **err)
 {
-  STThread *thread;
   int status;
 
   g_return_val_if_fail(transfer != NULL, FALSE);
   g_return_val_if_fail(transfer->url != NULL, FALSE);
   g_return_val_if_fail(transfer->session != NULL, FALSE);
-  /*
-   * libcurl assumes that eitheir CURLOPT_WRITEFUNCTION or
-   * CURLOPT_WRITEDATA will be set, so we require a body callback.
-   */
-  g_return_val_if_fail(transfer->body_cb != NULL, FALSE);
 
-  thread = st_thread_get();
+  transfer->thread = st_thread_get();
+  g_return_val_if_fail(transfer->thread != NULL, FALSE);
 
-  if (thread)
+  if (st_thread_is_aborted(transfer->thread))
+    return FALSE;
+
+  transfer->downloaded = 0;
+
+  GDK_THREADS_ENTER();
+  st_thread_set_progress(transfer->thread, 0);
+  gdk_flush();
+  GDK_THREADS_LEAVE();
+
+  if (! st_transfer_set_proxy_settings(transfer))
     {
       GDK_THREADS_ENTER();
-
-      if (thread->printable)
-	sg_printable_printf(thread->printable, _("Connecting to %s..."), transfer->url);
-
-      if (thread->progress_bar)
-	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(thread->progress_bar), 0);
-
+      st_thread_abort(transfer->thread);
+      gdk_flush();
       GDK_THREADS_LEAVE();
+      
+      return FALSE;		/* aborted */
     }
-  
-  st_transfer_set_proxy_settings(transfer);
+
+  GDK_THREADS_ENTER();
+  st_thread_printf(transfer->thread, _("Connecting to %s..."), transfer->url);
+  gdk_flush();
+  GDK_THREADS_LEAVE();
   
   curl_easy_setopt(transfer->session->handle, CURLOPT_USERAGENT, AGENT_STRING);
   curl_easy_setopt(transfer->session->handle, CURLOPT_URL, transfer->url);
@@ -440,53 +545,85 @@ st_transfer_perform (STTransfer *transfer, GError **err)
   curl_easy_setopt(transfer->session->handle, CURLOPT_WRITEDATA, transfer->body_data);
   curl_easy_setopt(transfer->session->handle, CURLOPT_NOPROGRESS, FALSE);
   curl_easy_setopt(transfer->session->handle, CURLOPT_PROGRESSFUNCTION, st_transfer_progress_cb);
-  curl_easy_setopt(transfer->session->handle, CURLOPT_PROGRESSDATA, thread);
+  curl_easy_setopt(transfer->session->handle, CURLOPT_PROGRESSDATA, transfer);
   curl_easy_setopt(transfer->session->handle, CURLOPT_ERRORBUFFER, transfer->error);
-  curl_easy_setopt(transfer->session->handle, CURLOPT_PROXY, transfer->proxy_url);
+  curl_easy_setopt(transfer->session->handle, CURLOPT_PROXYTYPE, transfer->proxy_type);
+  curl_easy_setopt(transfer->session->handle, CURLOPT_PROXY, transfer->proxy_server);
+  curl_easy_setopt(transfer->session->handle, CURLOPT_PROXYPORT, transfer->proxy_port);
   curl_easy_setopt(transfer->session->handle, CURLOPT_PROXYUSERPWD, transfer->proxy_userpwd);
-  curl_easy_setopt(transfer->session->handle, CURLOPT_PASSWDFUNCTION, st_transfer_password_prompt_cb);
-  curl_easy_setopt(transfer->session->handle, CURLOPT_PASSWDDATA, transfer);
   curl_easy_setopt(transfer->session->handle, CURLOPT_COOKIEFILE, "");
-
+  curl_easy_setopt(transfer->session->handle, CURLOPT_FOLLOWLOCATION, TRUE);
+  curl_easy_setopt(transfer->session->handle, CURLOPT_NOBODY, transfer->body_cb == NULL);
+  curl_easy_setopt(transfer->session->handle, CURLOPT_ENCODING, "");
+  curl_easy_setopt(transfer->session->handle, CURLOPT_NOSIGNAL, TRUE);
+  
   status = curl_easy_perform(transfer->session->handle);
   
   st_transfer_free_proxy_settings(transfer);
 
   if (status != CURLE_OK
       && status != CURLE_ABORTED_BY_CALLBACK
-      && status != CURLE_BAD_PASSWORD_ENTERED)
-    g_set_error(err, 0, 0, "%s", transfer->error);
+      && ! st_thread_is_aborted(transfer->thread))
+    g_set_error(err, 0, 0, "%s", transfer->error); /* failure which isn't abort */
 
   return status == CURLE_OK;
 }
 
-static void
+static gboolean
 st_transfer_set_proxy_settings (STTransfer *transfer)
 {
-  g_return_if_fail(transfer != NULL);
+  gboolean status = TRUE;
+
+  g_return_val_if_fail(transfer != NULL, FALSE);
+
+  transfer->proxy_type = 0;
+  transfer->proxy_server = NULL;
+  transfer->proxy_port = 0;
+  transfer->proxy_userpwd = NULL;
 
   G_LOCK(st_settings);
 
-  if (st_settings.proxy_enabled && st_settings.proxy_url)
-    transfer->proxy_url = g_strdup(st_settings.proxy_url);
-  else
-    transfer->proxy_url = NULL;
-
+  /*
+   * We have to set transfer->proxy_userpwd, because even if the proxy
+   * is disabled in streamtuner, libcurl might still use http_proxy,
+   * ftp_proxy, etc...
+   */
   if (st_settings.proxy_auth_enabled && st_settings.proxy_auth_name)
     {
-      transfer->proxy_auth_name = g_strdup(st_settings.proxy_auth_name);
-      transfer->proxy_userpwd = g_strconcat(st_settings.proxy_auth_name,
-					    ":",
-					    st_settings.proxy_auth_password ? st_settings.proxy_auth_password : "",
-					    NULL);
+      char *password;
+
+      if (st_settings.proxy_auth_password)
+	password = g_strdup(st_settings.proxy_auth_password);
+      else
+	{
+	  GDK_THREADS_ENTER();
+	  st_thread_printf(transfer->thread, _("Waiting for proxy password..."));
+	  password = st_transfer_password_prompt(st_settings.proxy_auth_name);
+	  gdk_flush();
+	  GDK_THREADS_LEAVE();
+	}
+
+      if (password)
+	{
+	  transfer->proxy_userpwd = g_strconcat(st_settings.proxy_auth_name, ":", password, NULL);
+	  st_transfer_zero_and_free(password);
+	}
+      else			/* cancelled by user */
+	status = FALSE;
     }
   else
-    {
-      transfer->proxy_auth_name = NULL;
-      transfer->proxy_userpwd = NULL;
+    transfer->proxy_userpwd = g_strdup(""); /* libcurl crashes in handleSock5Proxy() if this is NULL */
+
+  if (status && st_settings.proxy_enabled && st_settings.proxy_server)
+    { 
+      transfer->proxy_type = st_settings.proxy_type;
+      transfer->proxy_server = g_strdup(st_settings.proxy_server);
+      transfer->proxy_port = st_settings.proxy_port;
     }
 
   G_UNLOCK(st_settings);
+
+  return status;
 }
 
 static void
@@ -494,8 +631,7 @@ st_transfer_free_proxy_settings (STTransfer *transfer)
 {
   g_return_if_fail(transfer != NULL);
   
-  g_free(transfer->proxy_url);
-  st_transfer_zero_and_free(transfer->proxy_auth_name);
+  g_free(transfer->proxy_server);
   st_transfer_zero_and_free(transfer->proxy_userpwd);
 }
 
@@ -506,130 +642,74 @@ st_transfer_escape (const char *url)
 }
 
 static gboolean
-st_transfer_progress_cb (void *data,
+st_transfer_progress_cb (gpointer data,
 			 double dltotal,
 			 double dlnow,
 			 double ultotal,
 			 double ulnow)
 {
-  STThread *thread = data;
+  STTransfer *transfer = data;
   gboolean aborted = FALSE;
 
-  if (thread)
+  aborted = st_thread_is_aborted(transfer->thread);
+  if (! aborted)
     {
-      aborted = st_thread_is_aborted(thread);
+      GDK_THREADS_ENTER();
 
-      if (! aborted)
-	{
-	  GDK_THREADS_ENTER();
-
-	  if (thread->printable)
-	    {
-	      if (dltotal == 0)
-		sg_printable_printf(thread->printable,
-				    ngettext("Receiving (%u byte so far)...",
-					     "Receiving (%u bytes so far)...",
-					     (unsigned int) dlnow),
-				    (unsigned int) dlnow);
-	      else
-		sg_printable_printf(thread->printable,
-				    ngettext("Receiving (%u byte out of %u)...",
-					     "Receiving (%u bytes out of %u)...",
-					     (unsigned int) dlnow),
-				    (unsigned int) dlnow,
-				    (unsigned int) dltotal);
-	    }
+      if (dltotal == 0)
+	st_thread_printf(transfer->thread,
+			 ngettext("Receiving (%u byte so far)...",
+				  "Receiving (%u bytes so far)...",
+				  (unsigned int) dlnow),
+			 (unsigned int) dlnow);
+      else
+	st_thread_printf(transfer->thread,
+			 ngettext("Receiving (%u byte out of %u)...",
+				  "Receiving (%u bytes out of %u)...",
+				  (unsigned int) dlnow),
+			 (unsigned int) dlnow,
+			 (unsigned int) dltotal);
 	  
-	  if (thread->progress_bar && dlnow != thread->downloaded)
-	    {
-	      if (dltotal == 0)
-		gtk_progress_bar_pulse((GtkProgressBar *) thread->progress_bar);
-	      else
-		gtk_progress_bar_set_fraction((GtkProgressBar *) thread->progress_bar, dlnow / dltotal);
-	    }
-
-	  GDK_THREADS_LEAVE();
-
-	  thread->downloaded = dlnow;
+      if (dlnow != transfer->downloaded)
+	{
+	  st_thread_set_progress(transfer->thread, dltotal == 0 ? -1 : dlnow / dltotal);
+	  transfer->downloaded = dlnow;
 	}
+
+      gdk_flush();
+      GDK_THREADS_LEAVE();
     }
 
   return aborted;
 }
 
-static gboolean
-st_transfer_password_prompt_cb (void *data,
-				char *prompt,
-				char *buffer,
-				int buflen)
+static char *
+st_transfer_password_prompt (const char *username)
 {
-  STTransfer *transfer = data;
   GtkWidget *dialog;
   char *password = NULL;
-  gboolean status = TRUE;	/* TRUE == failure */
 
-  g_return_val_if_fail(transfer != NULL, TRUE);
+  g_return_val_if_fail(username != NULL, NULL);
 
-  GDK_THREADS_ENTER();
-
-  dialog = sgtk_auth_dialog_new(st_shell_get_transient(),
+  dialog = sgtk_auth_dialog_new(st_main_get_transient(),
 				_("Please enter your proxy password."));
 
-  sgtk_auth_dialog_set_name(SGTK_AUTH_DIALOG(dialog), transfer->proxy_auth_name);
+  sgtk_auth_dialog_set_name(SGTK_AUTH_DIALOG(dialog), username);
   sgtk_auth_dialog_set_name_sensitive(SGTK_AUTH_DIALOG(dialog), FALSE);
 
-  if (sgtk_dialog_run_from_thread(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
+  if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
     password = g_strdup(sgtk_auth_dialog_get_password(SGTK_AUTH_DIALOG(dialog)));
 
   gtk_widget_destroy(dialog);
 
-  GDK_THREADS_LEAVE();
-
-  if (password)
-    {
-      if (strlen(password) < buflen)
-	{
-	  strcpy(buffer, password);
-	  status = FALSE;
-	}
-
-      st_transfer_zero_and_free(password);
-    }
-  else
-    {
-      STThread *thread;
-
-      thread = st_thread_get();
-      if (thread)
-	{
-	  GDK_THREADS_ENTER();
-	  
-	  switch (thread->type)
-	    {
-	    case ST_THREAD_TYPE_STREAM:
-	      st_shell_stop_task(thread);
-	      break;
-	      
-	    case ST_THREAD_TYPE_REFRESH:
-	      st_browser_tab_stop(ST_BROWSER_TAB(thread->tab));
-	      break;
-
-	    default:
-	      g_assert_not_reached();
-	    }
-
-	  GDK_THREADS_LEAVE();
-	}
-    }
-  
-  return status;
+  return password;
 }
 
 static size_t
-st_transfer_session_get_cb (char *buffer,
+st_transfer_session_get_cb (const char *buffer,
 			    size_t size,
 			    size_t nitems,
-			    void *data)
+			    gpointer data)
 {
   GString *string = data;
   size_t len;
@@ -641,10 +721,25 @@ st_transfer_session_get_cb (char *buffer,
 }
 
 static size_t
-st_transfer_session_get_by_line_cb (char *buffer,
+st_transfer_session_get_binary_cb (const char *buffer,
+				   size_t size,
+				   size_t nitems,
+				   gpointer data)
+{
+  GByteArray *array = data;
+  size_t len;
+
+  len = size * nitems;
+  g_byte_array_append(array, buffer, len);
+  
+  return len;
+}
+
+static size_t
+st_transfer_session_get_by_line_cb (const char *buffer,
 				    size_t size,
 				    size_t nitems,
-				    void *data)
+				    gpointer data)
 {
   STTransferLineData *line_data = data;
   size_t len;
@@ -694,3 +789,16 @@ st_transfer_zero_and_free (char *buffer)
       g_free(buffer);
     }
 }
+
+/*** private implementation **************************************************/
+
+void
+st_transfer_init (void)
+{
+  if (curl_global_init(CURL_GLOBAL_ALL) != CURLE_OK)
+    {
+      st_error_dialog(_("A fatal error has occurred."),
+		      _("Unable to initialize the curl library."));
+      exit(1);
+    }
+}
